"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.trace = exports.setDebugger = exports.otelSdk = exports.loggingMiddleware = exports.logger = exports.DiagLogLevel = void 0;
const api_1 = require("@opentelemetry/api");
Object.defineProperty(exports, "trace", { enumerable: true, get: function () { return api_1.trace; } });
const api_2 = require("@opentelemetry/api");
Object.defineProperty(exports, "DiagLogLevel", { enumerable: true, get: function () { return api_2.DiagLogLevel; } });
const api_logs_1 = require("@opentelemetry/api-logs");
const auto_instrumentations_node_1 = require("@opentelemetry/auto-instrumentations-node");
const exporter_logs_otlp_http_1 = require("@opentelemetry/exporter-logs-otlp-http");
const exporter_metrics_otlp_http_1 = require("@opentelemetry/exporter-metrics-otlp-http");
const exporter_trace_otlp_http_1 = require("@opentelemetry/exporter-trace-otlp-http");
const resources_1 = require("@opentelemetry/resources");
const sdk_logs_1 = require("@opentelemetry/sdk-logs");
const sdk_metrics_1 = require("@opentelemetry/sdk-metrics");
const sdk_node_1 = require("@opentelemetry/sdk-node");
const sdk_trace_base_1 = require("@opentelemetry/sdk-trace-base");
const semantic_conventions_1 = require("@opentelemetry/semantic-conventions");
const dotenv_1 = require("dotenv");
const express_1 = __importDefault(require("express"));
(0, dotenv_1.config)();
// For troubleshooting, set the log level to DiagLogLevel.DEBUG
const setDebugger = (LogLevel) => api_2.diag.setLogger(new api_2.DiagConsoleLogger(), LogLevel);
exports.setDebugger = setDebugger;
const env = {
    nodeEnv: process.env.NODE_ENV || 'local',
    service: {
        name: process.env.SERVER_HOSTNAME || 'undefined service',
        version: process.env.npm_package_version || 'undefined service version',
    },
    collector: {
        url: process.env.OPENTELEMETRY_COLLECTOR_URL || 'http://otelcol:4318',
    },
};
const resource = new resources_1.Resource({
    [semantic_conventions_1.SemanticResourceAttributes.SERVICE_NAME]: env.service.name,
    [semantic_conventions_1.SemanticResourceAttributes.SERVICE_VERSION]: env.service.version,
});
const traceExporter = new exporter_trace_otlp_http_1.OTLPTraceExporter({
    url: `${env.collector.url}/v1/traces`,
});
const spanProcessor = new sdk_trace_base_1.BatchSpanProcessor(traceExporter);
const metricReader = env.nodeEnv !== 'local'
    ? new sdk_metrics_1.PeriodicExportingMetricReader({
        exporter: new exporter_metrics_otlp_http_1.OTLPMetricExporter({
            url: `${env.collector.url}/v1/metrics`,
        }),
        exportIntervalMillis: 10000,
    })
    : undefined;
const logRecordProcessor = env.nodeEnv === 'local'
    ? new sdk_logs_1.BatchLogRecordProcessor(new exporter_logs_otlp_http_1.OTLPLogExporter({
        url: `${env.collector.url}/v1/logs`,
    }))
    : undefined;
const otelSdk = new sdk_node_1.NodeSDK({
    resource,
    spanProcessor,
    metricReader,
    logRecordProcessor,
    instrumentations: [
        (0, auto_instrumentations_node_1.getNodeAutoInstrumentations)({
            '@opentelemetry/instrumentation-fs': {
                enabled: false,
            },
        }),
    ],
});
exports.otelSdk = otelSdk;
// gracefully shut down the SDK on process exit
process.on('SIGTERM', () => {
    otelSdk
        .shutdown()
        .then(() => console.log('Open Telemtry SDK shut down successfully'), (err) => console.log('Error shutting down Open Telemtry  SDK', err))
        .finally(() => process.exit(0));
});
const app = (0, express_1.default)();
/**
 * Open Telemtry Logging middleware
 */
const loggingMiddleware = app.use((req, res, next) => {
    try {
        const activeSpan = api_1.trace.getActiveSpan();
        const logger = api_logs_1.logs.getLogger(env.service.name);
        if (activeSpan !== undefined && logger !== undefined) {
            const temp = res.json;
            const log = ({ severityNumber, severityText, context, message }) => {
                const obj = {
                    severityNumber,
                    severityText,
                    body: JSON.stringify({ context, message }),
                };
                console.log('--- LOG ---', obj);
                logger.emit(obj);
            };
            res.json = function (...args) {
                log({
                    severityNumber: api_logs_1.SeverityNumber.INFO,
                    severityText: 'INFO',
                    context: 'express request log',
                    message: {
                        request: {
                            headers: req.headers,
                            body: req.body,
                            query: req.query,
                            params: req.params,
                        },
                        response: {
                            body: args['0'],
                        },
                    },
                });
                return temp.apply(this, args);
            };
        }
    }
    catch (e) {
        console.log('--- ERR :( ---', e);
    }
    next();
});
exports.loggingMiddleware = loggingMiddleware;
const logger = api_logs_1.logs.getLogger(env.service.name);
exports.logger = logger;
